# Copyright (c) Hegel AI, Inc.
# All rights reserved.
#
# This source code's license can be found in the
# LICENSE file in the root directory of this source tree.


class PromptToolsUtilityError(Exception):
    r"""
    An exception to throw when something goes wrong with the prompttools utility.
    """

    pass
