from unittest import TestCase

from prompttools.harness import (
    ChatHistoryExperimentationHarness,
    ChatModelComparisonHarness,
    PromptTemplateExperimentationHarness,
    SystemPromptExperimentationHarness,
)


class TestHarness(TestCase):
    # TODO: Currently, it only ensures importing is correct.
    #       Add unit tests to verify initialization.
    def test_chat_history_exp_harness(self):
        pass

    def test_chat_model_exp_harness(self):
        pass

    def test_prrmpt_template_exp_harness(self):
        pass

    def test_system_prompt_exp_harness(self):
        pass
